#!/usr/bin/perl
# $Id: starter_pdfa2b.pl,v 1.7 2013/01/29 13:10:53 rp Exp $
#
# PDF/A-2b starter:
# Create PDF/A-2b conforming output with layers, transparency and
# PDF/A attachments.
#
# Required software: PDFlib/PDFlib+PDI/PPS 9
# Required data: font file, image file, ICC output intent profile
#                (see www.pdflib.com for ICC profiles)

use PDFlib::PDFlib 9;
use strict;

# This is where the data files are. Adjust as necessary.
use constant searchpath => "../data";

use constant imagefile => "zebra.tif";
use constant attachments => ( "lionel.pdf", "nesrin.pdf" );

my $p = new PDFlib::PDFlib;

eval {
    # This means we must check return values of load_font() etc.
    $p->set_option("errorpolicy=return");

    # all strings are expected as utf8
    $p->set_option("stringformat=utf8");

    $p->set_option("SearchPath={{" . searchpath ."}}");

    # Initially display the layer panel and show the full page
    if ($p->begin_document("starter_pdfa2b.pdf",
        "openmode=layers viewerpreferences={fitwindow=true} pdfa=PDF/A-2b")
            == -1) {
        printf("Error: %s\n", $p->get_errmsg());
        exit(2);
    }

    $p->set_info("Creator", "PDFlib starter sample");
    $p->set_info("Title", "starter_pdfa2b");

    if ($p->load_iccprofile("ISOcoated.icc", "usage=outputintent") == -1)
    {
        printf("Error: %s\n", $p->get_errmsg());
        printf("Please install the ICC profile package from " .
               "www.pdflib.com to run the PDF/A-2b starter sample.\n");
        exit(2);
    }

    # Define the layers, with only English and image layers switched on.
    my $layer_english = $p->define_layer("English text", "");
    my $layer_german  = $p->define_layer("German text", "defaultstate=false");
    my $layer_french  = $p->define_layer("French text", "defaultstate=false");
    my $layer_image   = $p->define_layer("Images", "");

    # Define a radio button relationship for the language layers, so only
    # one language can be switched on at a time.
    my $optlist = sprintf
	    "group={%d %d %d}",
	    $layer_english, $layer_german, $layer_french;
    $p->set_layer_dependency("Radiobtn", $optlist);

    $p->begin_page_ext(595, 842, "");

    # Font embedding is required for PDF/A
    my $font = $p->load_font("LuciduxSans-Oblique", "unicode", "embedding");

    if ($font == -1) {
        printf("Error: %s\n", $p->get_errmsg());
        exit(2);
    }

    $optlist = sprintf("font=%d fontsize=24", $font);
    
    $p->begin_layer($layer_english);
    my $textflow = $p->create_textflow(
                "PDF/A-2b starter sample with layers, transparency " .
                "and attachments", $optlist);
    $p->fit_textflow($textflow, 50, 650, 550, 700, "");
    $p->delete_textflow($textflow);

    $p->begin_layer($layer_german);
    my $textflow = $p->create_textflow(
                "PDF/A-2b Starter-Beispiel mit Ebenen, Transparenz " .
                "und Anlagen", $optlist);
    $p->fit_textflow($textflow, 50, 650, 550, 700, "");
    $p->delete_textflow($textflow);

    $p->begin_layer($layer_french);
    my $textflow = $p->create_textflow(
                "PDF/A-2b starter exemple avec des calques, " .
                "de la transparence et des annexes", $optlist);
    $p->fit_textflow($textflow, 50, 650, 550, 700, "");
    $p->delete_textflow($textflow);

    $p->begin_layer($layer_image);

    $p->setfont($font, 48);

    # The RGB image needs an ICC profile; we use sRGB.
    my $icc = $p->load_iccprofile("sRGB", "");
    $optlist = sprintf "iccprofile=%d", $icc;
    my $image = $p->load_image("auto", imagefile, $optlist);

    if ($image == -1) {
        printf("Error: %s\n", $p->get_errmsg());
        exit(2);
    }

    my $width = $p->info_image($image, "width", "");
    my $height = $p->info_image($image, "height", "");

    # Place the image on the page and close it
    $p->fit_image($image, 0.0, 0.0, "");
    $p->close_image($image);

    # Set transparency in the graphics state
    my $gstate = $p->create_gstate("opacityfill=0.5");
    $p->set_gstate($gstate);

    # Place a transparent diagonal stamp across the image area, in
    # different colors
    $optlist = sprintf "boxsize={%f %f} stamp=ll2ur", $width, $height;
    
    $p->begin_layer($layer_english);
    $p->setcolor("fill", "Lab", 100, 28, 75, 0);
    $p->fit_textline("Transparent text", 0, 0, $optlist);

    $p->begin_layer($layer_german);
    $p->setcolor("fill", "Lab", 33.725, 5, -52, 0);
    $p->fit_textline("Transparenter Text", 0, 0, $optlist);

    $p->begin_layer($layer_french);
    $p->setcolor("fill", "Lab", 0, 0, 0, 0);
    $p->fit_textline("Texte transparent", 0, 0, $optlist);
        
    # Close all layers
    $p->end_layer();

    $p->end_page_ext("");
    
    # Construct option list with attachment handles. The attachments must
    # be PDF/A-1 or PDF/A-2 files.
    $optlist = "attachments={";
    foreach (attachments) {
        my $attachment_handle =
                $p->load_asset("Attachment", $_,
                            "description={This is a PDF/A attachment}");

        if ($attachment_handle == -1) {
            printf("Error loading attachment: %s\n", $p->get_errmsg());
            exit(2);
        }

        $optlist .= " " . $attachment_handle;
    }
    $optlist .= "}";

    $p->end_document($optlist);
};

if ($@) {
    die("$0: PDFlib Exception occurred:\n$@");
}
